package com.trafficparrot.sdk.jms;

import com.github.jknack.handlebars.Handlebars;
import com.github.jknack.handlebars.Helper;
import com.github.jknack.handlebars.Template;
import com.github.jknack.handlebars.helper.StringHelpers;
import com.trafficparrot.sdk.handlebars.TrafficParrotHelper;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static java.lang.String.format;
import static java.util.Arrays.asList;
import static java.util.Collections.emptyList;
import static java.util.Collections.emptyMap;
import static org.apache.commons.lang3.StringUtils.isBlank;

public class JmsHandlebarsResponseTransformer {
    private final Handlebars handlebars;
    private final boolean enabled;
    private final TpLogger logger;

    public JmsHandlebarsResponseTransformer(Handlebars handlebars, boolean enabled, TpLogger logger) {
        this.handlebars = handlebars;
        this.enabled = enabled;
        this.logger = logger;
    }

    public static JmsHandlebarsResponseTransformer handlebarsTransformer(String commaSeparatedHandlebarHelpers, Boolean handlebarsEnabled, TpLogger logger) {
        if (handlebarsEnabled) {
            return defineHelpers(toHelpers(parseList(commaSeparatedHandlebarHelpers)), logger);
        }
        return handlebarsAreDisabled(logger);
    }

    public static JmsHandlebarsResponseTransformer handlebarsAreDisabled(TpLogger logger) {
        return new JmsHandlebarsResponseTransformer(null, false, logger);
    }

    // TODO(4/5) copied from VirtualServiceServer
    private static Map<String, Helper> toHelpers(List<String> dynamicResponseTemplateExtensions) {
        Map<String, Helper> helpers = new HashMap<>();
        for (String extension : dynamicResponseTemplateExtensions) {
            TrafficParrotHelper helper = toHelper(extension);
            helpers.put(helper.getName(), helper);
        }
        return helpers;
    }

    // TODO(4/5) copied from VirtualServiceServer
    private static TrafficParrotHelper toHelper(String helperClassName) {
        try {
            return ((TrafficParrotHelper) Class.forName(helperClassName).newInstance());
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException e) {
            throw new IllegalArgumentException(e);
        }
    }

    // TODO(4/5) copied from VirtualServiceServer
    public static List<String> parseList(String propertyValue) {
        if (isBlank(propertyValue)) {
            return emptyList();
        }
        return asList(propertyValue.split(","));
    }

    public static JmsHandlebarsResponseTransformer defineHelpers(final Map<String, Helper> helpers, TpLogger logger) {
        Handlebars handlebars = new Handlebars();

        for (StringHelpers helper : StringHelpers.values()) {
            handlebars.registerHelper(helper.name(), helper);
        }

        for (Map.Entry<String, Helper> entry : helpers.entrySet()) {
            handlebars.registerHelper(entry.getKey(), entry.getValue());
        }

        handlebars.registerHelperMissing(new MissingHelper());

        return new JmsHandlebarsResponseTransformer(handlebars, true, logger);
    }

    public JmsResponse transform(JmsResponse response) {
        if (enabled) {
            try {
                Template responseTemplate = handlebars.compileInline(response.text);
                String responseTextWithTemplateApplied = responseTemplate.apply(emptyMap());
                return new JmsResponse(response.destination, response.jmsResponseTransformerClassName, responseTextWithTemplateApplied);
            } catch (com.github.jknack.handlebars.HandlebarsException | IOException e) {
                String errorMessage = format("Problem while applying handlebars: \n\n%s\n\nOriginal text: \n%s", e.getMessage(), response.text);
                logger.error(errorMessage, e);
                return new JmsResponse(response.destination, errorMessage + ". \n\nSee logs for more details.");
            }
        } else {
            return response;
        }
    }

}
