package com.trafficparrot.sdk.jms;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.collect.ComparisonChain;
import org.apache.commons.lang3.StringUtils;

import javax.jms.Message;

import static java.lang.String.format;
import static org.apache.commons.lang3.builder.ToStringBuilder.reflectionToString;
import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class JmsMessageMapping implements Comparable<JmsMessageMapping> {

    public final String mappingId;
    public final JmsRequestMatcher request;
    public final JmsResponse response;

    public JmsMessageMapping(@JsonProperty("mappingId") String mappingId,
                             @JsonProperty("request") JmsRequestMatcher request,
                             @JsonProperty("response") JmsResponse response) {
        this.mappingId = mappingId;
        this.request = request;
        this.response = response;
    }

    @JsonIgnore
    public JmsResponse transformedResponse(Message requestMessage, JmsHandlebarsResponseTransformer handlebars) {
        JmsResponse afterResponseTransformer = responseTransformerTransform(requestMessage);
        JmsResponse afterHandlebars = handlebars.transform(afterResponseTransformer);
        return afterHandlebars;
    }

    private JmsResponse responseTransformerTransform(Message requestMessage) {
        if (StringUtils.isNotBlank(response.jmsResponseTransformerClassName) && !JmsResponse.NO_TRANSFORMER.equals(response.jmsResponseTransformerClassName)) {
            JmsResponseTransformer jmsResponseTransformer = parseExtension(response.jmsResponseTransformerClassName);
            return jmsResponseTransformer.transform(request.destination, requestMessage, response);
        } else {
            return response;
        }
    }

    @JsonIgnore
    private static JmsResponseTransformer parseExtension(String extensionsPropertyValue) {
        try {
            return Class.forName(extensionsPropertyValue).asSubclass(JmsResponseTransformer.class).newInstance();
        } catch (ClassNotFoundException e) {
            throw new IllegalArgumentException(format("Extension '%s' was not found on the classpath", extensionsPropertyValue), e);
        } catch (IllegalAccessException | InstantiationException e) {
            throw new IllegalArgumentException(format("Extension '%s' must have a public zero argument constructor", extensionsPropertyValue), e);
        }
    }

    @Override
    @JsonIgnore
    public String toString() {
        return reflectionToString(this, SHORT_PREFIX_STYLE);
    }

    @Override
    public int compareTo(JmsMessageMapping other) {
        return ComparisonChain.start()
                .compare(mappingId, other.mappingId)
                .compare(request, other.request)
                .compare(response, other.response)
                .result();
    }
}
